/*- - - - - - - - - - - - - - - - - - - - - - - - - - - -
	CALEDIT.CPP

	TCalendarEdit Class

	Author:     Daniel BERMAN (CompuServe: 100567,3464)
	Date:       09 jully 1995 (1.0)
	Date:       13 jully 1995 (1.01)
	Date:       23 jully 1995 (1.02)
	Date:       16 august 1995 (1.03)

*/

#include <owl\owlpch.h>
#pragma hdrstop

#include <stdlib.h>

#include "intldate.h"
#include "caledit.h"
#include "caledit.rh"



class TPopupCalWin : public TWindow
{
public:

	TPopupCalWin(TWindow* parent, TCalendarEdit* edit, TModule* module = 0);
	~TPopupCalWin();

	void					SetupWindow();

	const TIntlDate&	GetDate();
	void					SetDate(const TIntlDate& date);

private:

	TFont* Font;
	int FontWidth;
	int FontHeight;

	TIntlDate Date;

	int MonthRow;
	int WeekDaysRow;
	int DaysRow;
	int StartCol;

	bool Dragging;
	TRect CurRect;

	TBitmap* BmpPrev;
	TBitmap* BmpNext;
	TRect BtnPrevRect;
	TRect BtnNextRect;

	TCalendarEdit* CalEdit;

	void DrawFocusRect(TDC& dc, int x, int y);
	void Paint(TDC& dc, bool erase, TRect& rect);
	void PaintBorders(TDC& dc, TRect& rect);

	void EvKeyDown(uint key, uint repeatCount, uint flags);
	void EvKillFocus(HWND hwndGetFocus);
	void EvLButtonDown(uint modKeys, TPoint& point);
	void EvLButtonUp(uint modKeys, TPoint& point);
	void EvMouseMove(uint modKeys, TPoint& point);

	DECLARE_RESPONSE_TABLE(TPopupCalWin);
};

class TCalendarBtn : public TButton
{
public:

	TCalendarBtn(TWindow *parent, TCalendarEdit* edit, TModule* module);
	~TCalendarBtn();

	char far* 			GetClassName();

	void					Layout();
	void					SetupWindow();

	const TIntlDate&	GetDate();
	void					SetDate(const TIntlDate& date);

   bool					IsMember(HWND hWnd);

	void					EvClicked();

private:

	TCalendarEdit* CalEdit;
	TPopupCalWin* CalWin;
	TBitmap* BmpUp;
	TBitmap* BmpDn;

	void EvDrawItem(uint ctrlId, DRAWITEMSTRUCT far& drawInfo);

	DECLARE_RESPONSE_TABLE(TCalendarBtn);
};


//-- TCalendarBtn --------------------------------------------------------------

DEFINE_RESPONSE_TABLE1(TCalendarBtn, TButton)
	EV_WM_DRAWITEM,
	EV_NOTIFY_AT_CHILD(BN_CLICKED, EvClicked),
END_RESPONSE_TABLE;

TCalendarBtn::TCalendarBtn(TWindow *parent, TCalendarEdit* edit, TModule* module)
	: TButton(parent, 0, "", 0, 0, 0, 0, false, module)
{
	Attr.Style |= BS_OWNERDRAW;
	Attr.Style &= ~WS_TABSTOP;

	CalEdit = edit;
	CalWin = new TPopupCalWin(Parent, edit, module);
	CalWin->DisableAutoCreate();

	BmpUp = new TBitmap(*GetApplication(), IDB_CALBTNUP);
	BmpDn = new TBitmap(*GetApplication(), IDB_CALBTNDN);

	if (TYPESAFE_DOWNCAST(Parent, TGadgetWindow))
		CalEdit->Attr.H = BmpUp->Height();
}

TCalendarBtn::~TCalendarBtn()
{
	delete BmpUp;
	delete BmpDn;
}

void TCalendarBtn::EvClicked()
{
	if (*CalWin)
		return;

	TRect r = CalEdit->GetWindowRect();
	CalWin->Attr.X = r.left;
	CalWin->Attr.Y = r.bottom;
	CalWin->Create();
}

void TCalendarBtn::EvDrawItem(uint /*ctrlId*/, DRAWITEMSTRUCT far& drawInfo)
{
	TDC dc(drawInfo.hDC);
	TMemoryDC memDC(dc);

	if (drawInfo.itemState & ODS_SELECTED)
		memDC.SelectObject(*BmpDn);
	else
		memDC.SelectObject(*BmpUp);
	dc.BitBlt(0, 0, Attr.W, Attr.H, memDC, 0, 0);
}

char far* TCalendarBtn::GetClassName()
{
	return "BUTTON";
}

const TIntlDate& TCalendarBtn::GetDate()
{
	return CalWin->GetDate();
}

bool TCalendarBtn::IsMember(HWND hWnd)
{
	return (hWnd == *this || hWnd == *CalWin);
}

void TCalendarBtn::Layout()
{
	TRect	rcEdit = CalEdit->GetClientRect();
	TRect	rcThis = rcEdit;

	rcThis.left	= rcThis.right - BmpUp->Width();
	rcEdit.right = rcThis.left;

	CalEdit->MapWindowPoints(*Parent, rcThis, 2);
	SetWindowPos(*CalEdit, rcThis, SWP_NOACTIVATE);
	CalEdit->SetWindowPos(NULL, 0, 0, rcEdit.Width(), rcEdit.Height(),
								 SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOZORDER);
}

void TCalendarBtn::SetDate(const TIntlDate& date)
{
	CalWin->SetDate(date);
}

void TCalendarBtn::SetupWindow()
{
	TButton::SetupWindow();
	Layout();
}


//-- TPopupCalWin --------------------------------------------------------------

DEFINE_RESPONSE_TABLE1(TPopupCalWin, TWindow)
	EV_WM_KEYDOWN,
	EV_WM_KILLFOCUS,
	EV_WM_LBUTTONDOWN,
	EV_WM_LBUTTONUP,
	EV_WM_MOUSEMOVE,
END_RESPONSE_TABLE;

TPopupCalWin::TPopupCalWin(TWindow* parent, TCalendarEdit* edit, TModule* module)
	: TWindow(parent, 0, module)
{
	Attr.Style = WS_POPUP | WS_BORDER | WS_VISIBLE;

	CalEdit = edit;

	Font = new TFont("MS Sans Serif", -8);
	{
		TRect rc;
		TScreenDC dc;
		dc.SelectObject(*Font);
		dc.DrawText("Abc", -1, rc, DT_CENTER | DT_CALCRECT);
		FontWidth = rc.Width();
		FontHeight = rc.Height();
	}

	MonthRow = 0;
	WeekDaysRow = MonthRow + FontHeight;
	DaysRow = WeekDaysRow + FontHeight + 4;
	StartCol = 4;

	Attr.W = (7 * FontWidth) + ( 2 * StartCol);
	Attr.H = DaysRow + (6 * FontHeight) + 4;

	Dragging = false;

	BmpPrev = new TBitmap(*GetApplication(), IDB_CALBTNPREV);
	BtnPrevRect = TRect(2, 2, BmpPrev->Width() + 2, BmpPrev->Height() + 2);

	BmpNext = new TBitmap(*GetApplication(), IDB_CALBTNNEXT);
	BtnNextRect.left = Attr.W - BmpNext->Width() - 4;
	BtnNextRect.top = 2;
	BtnNextRect.right = BtnNextRect.left + BmpNext->Width();
	BtnNextRect.bottom = BtnNextRect.top + BmpNext->Height();

	SetBkgndColor(TColor::LtGray);
}

TPopupCalWin::~TPopupCalWin()
{
	delete Font;
	delete BmpPrev;
	delete BmpNext;
}

void TPopupCalWin::DrawFocusRect(TDC& dc, int x, int y)
{
	TRect rect = TRect(StartCol + x * FontWidth,
							 DaysRow + y * FontHeight,
							 StartCol + x * FontWidth + FontWidth,
							 DaysRow + y * FontHeight + FontHeight);

	TIntlDate date(((y+1)*7) - (7-(x+1)) - (Date.WeekDay(1)-1), Date.Month(), Date.Year());
	if (!date.IsValid())
		return;

	CalEdit->SetText(date);

	dc.SaveDC();

	if (!CurRect.IsEmpty()) {
		dc.SelectObject(TBrush(TColor(TColor::LtGray)));
		dc.PatBlt(CurRect.left, CurRect.top, CurRect.Width() - 1, 1, PATCOPY);
		dc.PatBlt(CurRect.left, CurRect.top, 1, CurRect.Height() - 1, PATCOPY);
		dc.PatBlt(CurRect.right - 1, CurRect.top, 1, CurRect.Height(), PATCOPY);
		dc.PatBlt(CurRect.left, CurRect.bottom - 1, CurRect.Width(), 1, PATCOPY);
	}

	dc.SelectObject(TBrush(TColor(TColor::Gray)));
	dc.PatBlt(rect.left, rect.top, rect.Width() - 1, 1, PATCOPY);
	dc.PatBlt(rect.left, rect.top, 1, rect.Height() - 1, PATCOPY);
	dc.SelectObject(TBrush(TColor(TColor::White)));
	dc.PatBlt(rect.right - 1, rect.top, 1, rect.Height(), PATCOPY);
	dc.PatBlt(rect.left, rect.bottom - 1, rect.Width(), 1, PATCOPY);

	CurRect = rect;

	dc.RestoreDC();
}

void TPopupCalWin::EvKeyDown(uint key, uint repeatCount, uint flags)
{
	TIntlDate curDate = Date;

	switch (key) {
		case VK_DOWN:
			Date += 7;
			break;
		case VK_END:
			if (GetKeyState(VK_CONTROL) < 0)
				if (Date < TIntlDate(31, 12, Date.Year()))
					Date = TIntlDate(31, 12, Date.Year());
				else
					Date = TIntlDate(31, 12, Date.Year() + 1);
			else
				if (Date.DayOfMonth() < Date.DaysInMonth())
					Date = TIntlDate(Date.DaysInMonth(), Date.Month(), Date.Year());
			break;
		case VK_ESCAPE:
			CalEdit->SetFocus();
			CloseWindow();
			return;
		case VK_HOME:
			if (GetKeyState(VK_CONTROL) < 0)
				if (Date > TIntlDate(1, 1, Date.Year()))
					Date = TIntlDate(1, 1, Date.Year());
				else
					Date = TIntlDate(1, 1, Date.Year() - 1);
			else
				if (Date.DayOfMonth() > 1)
					Date = TIntlDate(1, Date.Month(), Date.Year());
			break;
		case VK_LEFT:
			--Date;
			break;
		case VK_MULTIPLY:
			Date = TIntlDate();
			break;
		case VK_NEXT:
			if (GetKeyState(VK_CONTROL) < 0) {
				if (Date.Month() == 2) {
					TIntlDate date(1, 2, Date.Year() + 1);
					if (Date.DayOfMonth() > date.DaysInMonth()) {
						Date = TIntlDate(date.DaysInMonth(), 2, date.Year());
						break;
					}
				}
				Date = TIntlDate(Date.DayOfMonth(), Date.Month(), Date.Year() + 1);
			}
			else {
				if (Date.Month() == 1)
					if (Date.DayOfMonth() > Date.DaysInMonth(2)) {
						Date = TIntlDate(Date.DaysInMonth(2), 2, Date.Year());
						break;
					}
				Date += Date.DaysInMonth();
			}
			break;
		case VK_PRIOR:
			if (GetKeyState(VK_CONTROL) < 0) {
				if (Date.Month() == 2) {
					TIntlDate date(1, 2, Date.Year() - 1);
					if (Date.DayOfMonth() > date.DaysInMonth()) {
						Date = TIntlDate(date.DaysInMonth(), 2, date.Year());
						break;
					}
				}
				Date = TIntlDate(Date.DayOfMonth(), Date.Month(), Date.Year() - 1);
			}
			else {
				int days = Date.DaysInMonth(Date.Month()-1 < 1 ? 12 : Date.Month()-1);
				if (days < Date.DayOfMonth())
					Date -= Date.DayOfMonth();
				else
					Date -= days;
			}
			break;
		case VK_RETURN:
		case VK_SPACE:
			CalEdit->SetText(Date);
			CalEdit->SetFocus();
			CloseWindow();
			return;
		case VK_RIGHT:
			++Date;
			break;
		case VK_UP:
			Date -= 7;
			break;
		default:
			TWindow::EvKeyDown(key, repeatCount, flags);
			return;
	};

	if (Date.Month() != curDate.Month() || Date.Year() != curDate.Year())
		Invalidate();
	else {
		TClientDC dc(*this);
		DrawFocusRect(dc, Date.WeekDay() - 1, (Date.DayOfMonth() + Date.WeekDay(1) - 2 ) / 7);
	}
}

void TPopupCalWin::EvKillFocus(HWND /*hwndGetFocus*/)
{
	CloseWindow();
}

void TPopupCalWin::EvLButtonDown(uint modKeys, TPoint& point)
{
	SetCapture();

	if (BtnPrevRect.Contains(point) || BtnNextRect.Contains(point))
		return;

	Dragging = true;
	EvMouseMove(modKeys, point);
}

void TPopupCalWin::EvLButtonUp(uint /*modKeys*/, TPoint& point)
{
	ReleaseCapture();

	if (Dragging) {
		Dragging = false;

		int x = ((point.x  - StartCol) / FontWidth) + 1;
		int y = ((point.y - DaysRow) / FontHeight) + 1;
		TIntlDate date((y*7) - (7-x) - (Date.WeekDay(1)-1), Date.Month(), Date.Year());
		if (!date.IsValid())
			return;

		CalEdit->SetText(date);
		CalEdit->SetFocus();

		CloseWindow();
	}
	else if (BtnPrevRect.Contains(point)) {
		int days = Date.DaysInMonth(Date.Month()-1 < 1 ? 12 : Date.Month()-1);
		if (days < Date.DayOfMonth())
			Date -= Date.DayOfMonth();
		else
			Date -= days;
		Invalidate();
	}
	else if (BtnNextRect.Contains(point)) {
		Date += Date.DaysInMonth();
		Invalidate();
	}
}

void TPopupCalWin::EvMouseMove(uint modKeys, TPoint& point)
{
	if (Dragging)
		if (TRect(StartCol, DaysRow, StartCol + 7 * FontWidth,
					 DaysRow + 6 * FontHeight).Contains(point)) {
			TClientDC dc(*this);
			DrawFocusRect(dc,
							  (point.x - StartCol) / FontWidth,
							  (point.y - DaysRow) / FontHeight);
		}
	TWindow::EvMouseMove(modKeys, point);
}

const TIntlDate& TPopupCalWin::GetDate()
{
	return Date;
}

void TPopupCalWin::Paint(TDC& dc, bool /*erase*/, TRect& /*rect*/)
{
	int i;
	TRect r;
	TIntlDate today;
	int weekday = Date.WeekDay(1);
	TRect clientRect = GetClientRect();
	TFont boldFont("MS Sans Serif", -8, 0, 0, 0, FW_BOLD);

	PaintBorders(dc, clientRect);

	dc.SetBkMode(TRANSPARENT);

	TMemoryDC memDC(dc);
	memDC.SelectObject(*BmpPrev);
	dc.BitBlt(BtnPrevRect, memDC, TPoint(0,0));
	memDC.SelectObject(*BmpNext);
	dc.BitBlt(BtnNextRect, memDC, TPoint(0,0));

	char buf[80];
	wsprintf(buf, "%s %d", Date.NameOfMonth(), Date.Year());
	r = TRect(0, MonthRow, clientRect.right, MonthRow + FontHeight);
	dc.SelectObject(boldFont);
	dc.DrawText(buf, -1, r, DT_CENTER | DT_VCENTER);
	dc.SelectObject(*Font);

	r = TRect(StartCol, WeekDaysRow, StartCol + FontWidth, WeekDaysRow + FontHeight);
	for (i = 0; i < 7; i++) {
		dc.DrawText(Date.ShortDayName(i+1), -1, r, DT_CENTER | DT_VCENTER);
		r.left += FontWidth;
		r.right += FontWidth;
	}

	dc.SaveDC();
	clientRect.top = WeekDaysRow + FontHeight;
	clientRect.bottom = clientRect.top + 2;
	dc.SelectObject(TBrush(TColor(TColor::Gray)));
	dc.PatBlt(clientRect.left, clientRect.top, clientRect.Width() - 1, 1, PATCOPY);
	dc.SelectObject(TBrush(TColor(TColor::White)));
	dc.PatBlt(clientRect.left, clientRect.bottom - 1, clientRect.Width(), 1, PATCOPY);
	dc.RestoreDC();

	r = TRect(StartCol + ((weekday -1) * FontWidth), DaysRow, StartCol + ((weekday -1) * FontWidth) + FontWidth, DaysRow + FontHeight);
	for (i = 1; i <= Date.DaysInMonth(); i++, weekday++) {
		char buf[3];
		if (today == TIntlDate(i, Date.Month(), Date.Year()))
			dc.SelectObject(boldFont);
		dc.DrawText(itoa(i, buf, 10), -1, r, DT_CENTER | DT_VCENTER);
		if (today == TIntlDate(i, Date.Month(), Date.Year()))
			dc.SelectObject(*Font);
		r.left += FontWidth;
		r.right += FontWidth;
		if (weekday % 7 == 0) {
			r.left = StartCol;
			r.right = StartCol + FontWidth;
			r.top += FontHeight;
			r.bottom += FontHeight;
		}
	}

	DrawFocusRect(dc, Date.WeekDay()-1, (Date.DayOfMonth()+Date.WeekDay(1)-2)/7);
}

void TPopupCalWin::PaintBorders(TDC& dc, TRect& rect)
{
	dc.SaveDC();
	dc.SelectObject(TBrush(TColor(TColor::White)));
	dc.PatBlt(rect.left, rect.top, rect.Width() - 1, 1, PATCOPY);
	dc.PatBlt(rect.left, rect.top, 1, rect.Height() - 1, PATCOPY);
	dc.SelectObject(TBrush(TColor(TColor::Gray)));
	dc.PatBlt(rect.right - 1, rect.top, 1, rect.Height(), PATCOPY);
	dc.PatBlt(rect.left, rect.bottom - 1, rect.Width(), 1, PATCOPY);
	dc.RestoreDC();
}

void TPopupCalWin::SetDate(const TIntlDate& date)
{
	Date = date;
}

void TPopupCalWin::SetupWindow()
{
	TWindow::SetupWindow();

	char str[80];
	CalEdit->GetText(str, sizeof(str));

	Date = TIntlDate(str);
	if (!Date.IsValid())
		Date = TIntlDate();

	TRect rc, rcThis = GetWindowRect();
	::GetWindowRect(GetDesktopWindow(), &rc);
	if (rcThis.left < rc.left)
		rcThis.Offset(rc.left-rcThis.left, 0);
	if (rcThis.bottom > rc.bottom)
		rcThis.Offset(0, rc.bottom-rcThis.bottom);
	if (rcThis.right > rc.right)
		rcThis.Offset(rc.right-rcThis.right, 0);
	MoveWindow(rcThis);
}


//-- TCalendarEdit -------------------------------------------------------------

DEFINE_RESPONSE_TABLE1(TCalendarEdit, TEdit)
	EV_WM_CHAR,
	EV_WM_KILLFOCUS,
	EV_WM_KEYDOWN,
	EV_WM_SIZE,
END_RESPONSE_TABLE;

TCalendarEdit::TCalendarEdit(TWindow* parent, int id, const char far* text,
									  int x, int y, int w, int h, uint textLen,
									  TModule* module)
	: TEdit(parent, id, text, x, y, w, h, textLen, false, module)
{
	Init();
}

TCalendarEdit::TCalendarEdit(TWindow* parent, int resourceId, uint textLen,
									  TModule* module)
	: TEdit(parent, resourceId, textLen, module)
{
	Init();
}

TCalendarEdit::TCalendarEdit(TGadgetWindow* parent, int id, int w)
	: TEdit(parent, id, "", 0, 0, w, 0)
{
	Init();
}

TCalendarEdit::~TCalendarEdit()
{
	if (Font)
		delete Font;
}

void TCalendarEdit::Init()
{
	Font = 0;
	CalBtn = new TCalendarBtn(Parent, this, GetModule());
}

const TIntlDate& TCalendarEdit::GetDate()
{
	return CalBtn->GetDate();
}

void TCalendarEdit::EvChar(uint key, uint repeatCount, uint flags)
{
	if (key != '+' && key != '-')
		TEdit::EvChar(key, repeatCount, flags);
}
void TCalendarEdit::EvKeyDown(uint key, uint repeatCount, uint flags)
{
	switch (key) {
		case VK_ADD:
		case VK_SUBTRACT: {
			char str[80];
			GetText(str, sizeof(str));
			TIntlDate date(str);
			if (!date.IsValid())
				date = TIntlDate();
			key == VK_ADD ? ++date : --date;
			SetText(date);
			break;
		}
		case VK_DOWN:
			CalBtn->EvClicked();
			break;
		default:
			TEdit::EvKeyDown(key, repeatCount, flags);
	}
}

void TCalendarEdit::EvKillFocus(HWND hWndGetFocus)
{
	if (CalBtn->IsMember(hWndGetFocus))
		DefaultProcessing();
	else {
		char str[80];
		GetText(str, sizeof(str));
		TIntlDate date(str);
		SetText(date);
		TEdit::EvKillFocus(hWndGetFocus);
	}
}

void TCalendarEdit::EvSize(uint /*sizeType*/, TSize& /*size*/)
{
	static int bEvSize = false;

	if (!bEvSize) {
		bEvSize = true;
		CalBtn->Layout();
		DefaultProcessing();
		bEvSize = false;
	}
}

TIntlDate::HowToSpeak TCalendarEdit::SetLanguage(TIntlDate::HowToSpeak l)
{
	return TIntlDate::SetLanguageOption(l);
}

void TCalendarEdit::SetText(const TIntlDate& date)
{
	if (date.IsValid()) {
		CalBtn->SetDate(date);
		TEdit::SetText(date.AsString().c_str());
	}
	else
   	Clear();
}

void TCalendarEdit::SetText(const char far* str)
{
	SetText(TIntlDate(str));
}

void TCalendarEdit::SetupWindow()
{
	TEdit::SetupWindow();

	if (TYPESAFE_DOWNCAST(Parent, TGadgetWindow)) {
		Font = new TFont("MS Sans Serif", -8);
		SetWindowFont(*Font, false);
	}
}


//-- TDateValidator ------------------------------------------------------------

TDateValidator::TDateValidator()
	: TValidator()
{
}

void TDateValidator::Error()
{
	int stringId;
	TApplication* app = GetApplicationObject();

	switch (TIntlDate::GetLanguageOption()) {
		case TIntlDate::French:
			stringId = IDS_FRENCH_ERRMSG;
			break;
		case TIntlDate::English:
		default:
			stringId = IDS_ENGLISH_ERRMSG;
	}
	string msg = app->LoadString(stringId);

	TWindow* w = GetWindowPtr(app->GetMainWindow()->GetLastActivePopup());
	if (w)
		w->MessageBox(msg.c_str(), app->GetName(), MB_ICONEXCLAMATION | MB_OK);
	else
		::MessageBox(0, msg.c_str(), app->GetName(), MB_ICONEXCLAMATION | MB_OK | MB_TASKMODAL);
}

bool TDateValidator::IsValid(const char far* str)
{
	if (!*str)
		return true;

	TIntlDate date(str);
	bool isValid = date.IsValid();

	return isValid;
}


