/*- - - - - - - - - - - - - - - - - - - - - - - - - - - -
	INTLDATE.CPP

	TIntlDate Class

	Author:     Daniel BERMAN (CompuServe: 100567,3464)
	Date:       13 jully 1995 (1.01)
	Date:       23 jully 1995 (1.02)
	Date:       16 august 1995 (1.03)

	Most of the code is taken from Borland's TDate class.

*/

#include <stdio.h>
#include <time.h>

#include "intldate.h"



static const unsigned char daysInMonth[12] =
	{31,28,31,30,31,30,31,31,30,31,30,31};

static const char* WeekDayNames[][7] =
	{
		{"Monday","Tuesday","Wednesday","Thursday","Friday","Saturday","Sunday"},
		{"lundi","mardi","mercredi","jeudi","vendredi","samedi","dimanche"}
	};

static const char* ShortWeekDayNames[][7] =
	{
		{"Mo","Tu","We", "Th","Fr","Sa","Su"},
		{"Lu","Ma","Me","Je","Ve","Sa","Di"}
	};

static const char* MonthNames[][12] =
	{
		{"January","February","March","April","May","June",
		 "July","August","September","October","November","December"},
		{"janvier","fvrier","mars","avril","mai","juin",
		 "juillet","aot","septembre","octobre","novembre","dcembre"}
	};

static const julTy Jul1901 = 2415386L;  // Julian day for 1/1/1901


int TIntlDate::DateOrder = 0;
char TIntlDate::DateSeparator[2] = "";
char TIntlDate::ShortDateFmt[11] = "";

TIntlDate::HowToSpeak TIntlDate::Language = TIntlDate::English;



// Constructor for today's date
TIntlDate::TIntlDate()
{
	time_t timer = time(0);
	struct tm *now = localtime(&timer);
	JulNum = Jday(now->tm_mday, now->tm_mon+1, now->tm_year+1900);
}

// Constructor for the given day, month, and year.
TIntlDate::TIntlDate(dayTy day, monthTy month, yearTy year)
{
	JulNum = Jday(day, month, year);
}

// Return a string for this TIntlDate according to the [Intl] section in win.ini
string TIntlDate::AsString() const
{
	const char* picture;

	char buf[80];
	ostrstream strtemp(buf, sizeof(buf));
	strtemp.fill('0');

	SetIntlOptions();
	picture = ShortDateFmt;
	while (*picture) {
		switch (*picture) {
			case 'd':
			case 'D': {
				string tmp;
				int length = 0;
				while (*picture == 'd' || *picture == 'D') {
					tmp += picture++;
					length++;
				}
				strtemp.width(length);
				strtemp << DayOfMonth();
				break;
			}
			case 'm':
			case 'M': {
				string tmp;
				int length = 0;
				while (*picture == 'm' || *picture == 'M') {
					tmp += picture++;
					length++;
				}
				strtemp.width(length);
				strtemp << Month();
				break;
			}
			case 'y':
			case 'Y': {
				string tmp;
				int length = 0;
				while (*picture == 'y' || *picture == 'Y') {
					tmp += picture++;
					length++;
				}
				if (length == 2)
					strtemp << (Year() % 100);
				else
					strtemp << Year();
				break;
			}
			default: {
				strtemp << *picture;
				picture++;
			}
		}
	}

	strtemp << ends;
	return string(buf);
}

// Returns a string name for the day of the week,
// where Monday is 1 and Sunday is 7.
// Returns 0 for an invalid weekDayNumber.
const char* TIntlDate::DayName(dayTy weekDayNumber)
{
	if (AssertWeekDayNumber(weekDayNumber))
		return WeekDayNames[Language][weekDayNumber-1];
	else
		return 0;
}

// Returns the day name for a weekday number.
// Monday == 1, ... , Sunday == 7
// Return 0 for weekday number out of range
// Returns the day of the month of this TIntlDate
dayTy TIntlDate::DayOfMonth() const
{
	dayTy d; monthTy m; yearTy y;
	Mdy(d, m, y);
	return d;
}

// Is a day (1-31) within a given month?
int TIntlDate::DayWithinMonth(dayTy day, monthTy month, yearTy year)
{
	if (day <= 0 || !AssertIndexOfMonth(month))
		return 0;

	unsigned d = daysInMonth[month-1];
	if (LeapYear(year) && month == 2)
		d++;

	return day <= d;
}

// Return the number of days of a given month
// Return 0 if "month" is outside of the range 1 through 12, inclusive.
int TIntlDate::DaysInMonth(monthTy month) const
{
	unsigned days = 0;

	if (month == 0)
		month = Month();

	if (AssertIndexOfMonth(month)) {
		days = daysInMonth[month-1];
		if(Leap() && month == 2)
		days++;
	}

	return days;
}

TIntlDate::HowToSpeak TIntlDate::GetLanguageOption()
{
	return Language;
}

monthTy TIntlDate::Month() const
{
	dayTy d; monthTy m; yearTy y;
	Mdy(d, m, y);
	return m;
}

// Returns the string name for the given monthNumber (1-12).
const char* TIntlDate::MonthName(monthTy monthNumber)
{
	if (AssertIndexOfMonth(monthNumber))
		return MonthNames[Language][monthNumber-1];
	else
		return 0;
}

// Convert a string to a date according to the [Intl] section in win.ini
void TIntlDate::ParseFrom(const string& strDate)
{
	int val1, val2, val3;
	size_t pos1, pos2;

	JulNum = 0;

	SetIntlOptions();

	if ((pos1 = strDate.find_first_of("0123456789")) == NPOS)
		val1 = 0;
	else if ((pos2 = strDate.find_first_not_of("0123456789", pos1)) == NPOS)
		val1 = 0;
	else
		val1 = atoi(string(strDate, pos1, pos2).c_str());

	if ((pos1 = strDate.find_first_of("0123456789", pos2)) == NPOS)
		val2 = 0;
	else if ((pos2 = strDate.find_first_not_of("0123456789", pos1)) == NPOS)
		val2 = 0;
	else
		val2 = atoi(string(strDate, pos1, pos2).c_str());

	if ((pos1 = strDate.find_first_of("0123456789", pos2)) == NPOS)
		val3 = 0;
	else if ((pos2 = strDate.find_first_not_of("0123456789", pos1)) == NPOS)
		val3 = 0;
	else
		val3 = atoi(string(strDate, pos1, pos2).c_str());

	switch (DateOrder) {
		case 2:	// Year, Month, Day
			JulNum = Jday(val3, val2, val1);
			break;
		case 1:	// Day, Month, Year
			JulNum = Jday(val1, val2, val3);
			break;
		case 0:	// Month, Day, Year
		default:
			JulNum = Jday(val2, val1, val3);
			break;
	}
	if (JulNum < 1)
		JulNum = 0;
}

void TIntlDate::SetIntlOptions()
{
	DateOrder = GetProfileInt("intl", "iDate",  0);
	GetProfileString("intl", "sDate",  "/", DateSeparator, sizeof(DateSeparator));
	GetProfileString("intl", "sShortDate",  "MM/dd/yy", ShortDateFmt, sizeof(ShortDateFmt));
}

TIntlDate::HowToSpeak TIntlDate::SetLanguageOption(HowToSpeak l)
{
	HowToSpeak old = Language;
	Language = l;
	return old;
}

const char*	TIntlDate::ShortDayName(dayTy weekDayNumber)
{
	if (AssertWeekDayNumber(weekDayNumber))
		return ShortWeekDayNames[Language][weekDayNumber-1];
	else
		return 0;
}

// Returns 1 (Monday) through 7 (Sunday).
dayTy TIntlDate::WeekDay(dayTy day) const
{
	julTy julnum = (day == 0 ? JulNum : Jday(day, Month(), Year()));
	return dayTy(((((julnum+1)%7)+6)%7)+1);
}

yearTy TIntlDate::Year() const
{
	dayTy d; monthTy m; yearTy y;
	Mdy(d, m, y);
	return y;
}


//-- Algorithm from Communications of the ACM. ---------------------------------


// Convert Gregorian calendar date to the corresponding Julian day number j.
// Algorithm 199 from Communications of the ACM.
// Gregorian calendar started on Sep. 14, 1752.
// This function not valid before that.
// Returns 0 if the date is invalid.
julTy TIntlDate::Jday(dayTy d, monthTy m, yearTy y) const
{
	unsigned long c, ya;

	if (y <= 99)
		y += 1900;
	if (!DayWithinMonth(d, m, y))
		return (julTy)0;

	if (m > 2)
		m -= 3;
	else {
		m += 9;
		y--;
	}

	c = y / 100;
	ya = y - 100*c;

	return ((146097L*c)>>2) + ((1461*ya)>>2) + (153*m + 2)/5 + d + 1721119L;
}

// Algorithm from K & R, "The C Programming Language", 1st ed.
int TIntlDate::LeapYear(yearTy year)
{
	return (year&3) == 0 && year%100 != 0 || year % 400 == 0;
}

// Convert a Julian day number to its corresponding Gregorian calendar date.
// Algorithm 199 from Communications of the ACM.
// Gregorian calendar started on Sep. 14, 1752.
// This function not valid before that.
void TIntlDate::Mdy(dayTy& day, monthTy& month, yearTy& year) const
{
	unsigned long d;
	julTy j = JulNum - 1721119L;

	year  = (yearTy) (((j<<2) - 1) / 146097L);
	j     = (j<<2) - 1 - 146097L*year;
	d     = (j>>2);
	j     = ((d<<2) + 3) / 1461;
	d     = (d<<2) + 3 - 1461*j;
	d     = (d + 4)>>2;
	month = (monthTy)(5*d - 3)/153;
	d     = 5*d - 3 - 153*month;
	day   = (dayTy)((d + 5)/5);
	year  = (yearTy)(100*year + j);

	if (month < 10)
		month += 3;
	else {
		month -= 9;
		year++;
	}
}


