/*- - - - - - - - - - - - - - - - - - - - - - - - - - - -
	INTLDATE.H

	TIntlDate Class

	Author:     Daniel BERMAN (CompuServe: 100567,3464)
	Date:       13 jully 1995 (1.01)
	Date:       23 jully 1995 (1.02)
	Date:       16 august 1995 (1.03)

	Most of the code is taken from Borland's TDate class.

*/

#ifndef _TIntlDate_h
#define _TIntlDate_h

#include <cstring.h>
#include <strstrea.h>


typedef unsigned int dayTy;
typedef unsigned int monthTy;
typedef unsigned int yearTy;
typedef unsigned long julTy;


class TIntlDate
{
public:

	enum HowToSpeak {English, French};

	TIntlDate();
	TIntlDate(const char* str);
	TIntlDate(dayTy day, monthTy month, yearTy year);

	string					AsString() const;

	dayTy						DayOfMonth() const;
	monthTy					Month() const;
	yearTy					Year() const;

	dayTy						WeekDay(dayTy day = 0) const;

	int						IsValid() const;

	int						Leap() const;
	static int				LeapYear(yearTy year);

	int						DaysInMonth(monthTy month = 0) const;
	static int				DayWithinMonth(dayTy day, monthTy month, yearTy year);

	const char*				NameOfDay() const;
	const char*				NameOfMonth() const;

	static const char*	DayName(dayTy weekDayNumber);
	static const char*	ShortDayName(dayTy weekDayNumber);
	static const char*	MonthName(monthTy monthNumber);

	static HowToSpeak		SetLanguageOption(HowToSpeak l);
	static HowToSpeak		GetLanguageOption();

	int operator != ( const TIntlDate& date ) const;
	int operator <  ( const TIntlDate& date ) const;
	int operator <= ( const TIntlDate& date ) const;
	int operator == ( const TIntlDate& date ) const;
	int operator >  ( const TIntlDate& date ) const;
	int operator >= ( const TIntlDate& date ) const;

	julTy operator - (const TIntlDate& dt) const;
	friend TIntlDate operator + (const TIntlDate& dt, int dd);
	friend TIntlDate operator + (int dd, const TIntlDate& dt);
	friend TIntlDate operator - (const TIntlDate& dt, int dd);

	void operator ++ ();
	void operator -- ();
	void operator += (int dd);
	void operator -= (int dd);

	friend ostream& operator << (ostream& s, const TIntlDate& d);
	friend istream& operator >> (istream& s, TIntlDate& d);

protected:

	void					ParseFrom(const string& strDate);

	static void			SetIntlOptions();

	static int			AssertWeekDayNumber(dayTy day);
	static inline int	AssertIndexOfMonth(monthTy month);

private:

	julTy JulNum;

	static int	DateOrder;
	static char DateSeparator[2];
	static char ShortDateFmt[11];

	static HowToSpeak Language;

	TIntlDate(julTy j);

	julTy	Jday(dayTy day, monthTy month, yearTy year) const;
	void	Mdy(dayTy& day, monthTy& month, yearTy& year) const;
};


// Constructor for string date.
inline TIntlDate::TIntlDate(const char* str)
{
	ParseFrom(str);
}

// Constructor for Julian day.
inline TIntlDate::TIntlDate(julTy j)
{
	JulNum = j;
}

inline int TIntlDate::AssertIndexOfMonth(monthTy month)
{
	return month >= 1 && month <= 12;
}

inline int TIntlDate::AssertWeekDayNumber(dayTy day)
{
	return day >= 1 && day <= 7;
}

inline int TIntlDate::IsValid() const
{
	return JulNum > 0;
}

inline int TIntlDate::Leap() const
{
	return LeapYear(Year());
}

inline const char* TIntlDate::NameOfDay() const
{
	return DayName(WeekDay());
}

inline const char* TIntlDate::NameOfMonth() const
{
	return MonthName(Month());
}

inline int TIntlDate::operator != (const TIntlDate& date) const
{
	return JulNum != date.JulNum;
}

inline int TIntlDate::operator < (const TIntlDate& date) const
{
	return JulNum < date.JulNum;
}

inline int TIntlDate::operator <= (const TIntlDate& date) const
{
	return JulNum <= date.JulNum;
}

inline int TIntlDate::operator > (const TIntlDate& date) const
{
	return JulNum > date.JulNum;
}

inline int TIntlDate::operator == (const TIntlDate& date) const
{
	return JulNum == date.JulNum;
}

inline int TIntlDate::operator >= (const TIntlDate& date) const
{
	return JulNum >= date.JulNum;
}

inline julTy TIntlDate::operator - (const TIntlDate& dt) const
{
	return JulNum - dt.JulNum;
}

inline TIntlDate operator - (const TIntlDate& dt, int dd)
{
	return TIntlDate(dt.JulNum - dd);
}

inline TIntlDate operator + (const TIntlDate& dt, int dd)
{
	return TIntlDate(dt.JulNum + dd);
}

inline TIntlDate operator + (int dd, const TIntlDate& dt)
{
	return TIntlDate(dt.JulNum + dd);
}

inline void TIntlDate::operator ++ ()
{
	JulNum += 1;
}

inline void TIntlDate::operator -- ()
{
	JulNum -= 1;
}

inline void TIntlDate::operator += (int dd)
{
	JulNum += dd;
}

inline void TIntlDate::operator -= (int dd)
{
	JulNum -= dd;
}

inline ostream& operator << (ostream& s, const TIntlDate& d)
{
	s << d.AsString().c_str();
	return s;
}

inline istream& operator >> (istream& s, TIntlDate& d)
{
	string str;
	s >> str;
	d.ParseFrom(str);
	return s;
}


#endif

